#include <AppKit/AppKit.h>
#include "EvKController.h"
#include "EvKView.h"

@implementation EvKController

/*
 * The following a delegated notifications from NSWindow.
 * For each of them [notif object] provides the window.
 */
- (void)windowWillMove:(NSNotification *)notif
{
	NSWindow *window = [notif object];
	NSRect wFrame = [window frame];
	NSLog(@"win WillMove %@",
		NSStringFromPoint(wFrame.origin));
}

- (void)windowDidMove:(NSNotification *)notif
{
	NSWindow *window = [notif object];
	NSRect wFrame = [window frame];
	NSLog(@"win DidMove %@",
		NSStringFromPoint(wFrame.origin));
}

- (void)windowDidBecomeKey:(NSNotification *)notif
{
	NSLog(@"win DidBK");
}

- (void)windowDidResignKey:(NSNotification *)notif
{
	NSLog(@"win DidRK");
}

- (void)windowDidBecomeMain:(NSNotification *)notif
{
	NSLog(@"win DidBM");
}

- (void)windowDidResignMain:(NSNotification *)notif
{
	NSLog(@"win DidRM");
}

- (void)windowWillMiniaturize:(NSNotification *)notif
{
	NSLog(@"win WillMin");
}

- (void)windowDidMiniaturize:(NSNotification *)notif
{
	NSLog(@"win DidMin");
}

- (void)windowDidDeminiaturize:(NSNotification *)notif
{
	NSLog(@"win DidDemin");
}

- (void)windowWillClose:(NSNotification *)notif
{
	NSLog(@"win WillClose");
}

- (void)windowDidChangeScreen:(NSNotification *)notif
{
	NSLog(@"win DidCS");
}

/* A portion of a non-retained window has been exposed. */
- (void)windowDidExpose:(NSNotification *)notif
{
	NSDictionary *dict = [notif userInfo];
	NSValue *value = [dict objectForKey: @"NSExposedRect"];
	NSRect rect = [value rectValue];
	NSLog(@"win DidEx %@",
		NSStringFromRect(rect));
}

- (void)windowDidResize:(NSNotification *)notif
{
	NSWindow *window = [notif object];
	NSRect wFrame = [window frame];
	NSView *content = [window contentView];
	NSRect cFrame = [content frame];
	NSLog(@"win DidResize c%@ w%@",
		NSStringFromSize(cFrame.size),
		NSStringFromSize(wFrame.size));
}

#if 0 /* now noisy on gnustep */
- (void)windowDidUpdate:(NSNotification *)notif
{
	NSLog(@"win DidUpdate");
}
#endif


/*
 * The following are direct delegations from NSWindow.
 */
- (BOOL)windowShouldClose:(id)sender
{
	NSLog(@"win ShouldClose?");
	return YES;
}

- (NSSize)windowWillResize:(NSWindow *)sender
		    toSize:(NSSize)frameSize
{
	NSLog(@"win WillResize %@",
		NSStringFromSize(frameSize));
	return frameSize;
}

#if 0 /* very noisy on osx */
- (id)windowWillReturnFieldEditor:(NSWindow *)sender
			 toObject:(id)anObject
{
	NSLog(@"win WillReturnFieldEditor");
	return nil;
}
#endif

/* ======== End of Delegated methods ======== */


#define WW_MASK	((unsigned)(\
		 NSTitledWindowMask|\
		 NSClosableWindowMask|\
		 NSMiniaturizableWindowMask|\
		 NSResizableWindowMask|\
		 0)\
		)
-(id)init
{
	NSRect rect;
	NSView *view;

	rect = NSMakeRect(100,100, 100,100);

	_window = [[NSWindow alloc]
		initWithContentRect:	rect
			  styleMask:	WW_MASK
			    backing:	NSBackingStoreBuffered
			      defer:	NO];
	if (!_window)
		return nil;
	[_window setDelegate:self];
	[_window setTitle: @"Keyboard Events"];

	rect.origin = NSZeroPoint;
	view = [[EvKView alloc] initWithFrame:rect];
	if (!view)
		return nil;
	NSLog(@"ref = %u", [view retainCount]);
	[_window setContentView:view];
	NSLog(@"ref = %u", [view retainCount]);
	[view release];
	NSLog(@"ref = %u", [view retainCount]);


	[_window makeFirstResponder:view];
	[_window orderFront:nil];
	//[_window makeKeyAndOrderFront:nil];

	return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[_window setDelegate:nil];
	[_window release];

	[super dealloc];
}
@end
