#include <AppKit/AppKit.h>
#include "EvKView.h"
#ifdef __NeXT__
#include <AppKit/event.h>
#endif

#if defined(NX_DEVICELCTLKEYMASK)
#	define dd_LCTL	NX_DEVICELCTLKEYMASK
#elif defined(NX_NEXTCTRLKEYMASK)
#	define dd_LCTL	NX_NEXTCTRLKEYMASK
#endif
#if defined(NX_DEVICELSHIFTKEYMASK)
#	define dd_LSHFT	NX_DEVICELSHIFTKEYMASK
#elif defined(NX_NEXTLSHIFTKEYMASK)
#	define dd_LSHFT	NX_NEXTLSHIFTKEYMASK
#endif
#if defined(NX_DEVICELCMDKEYMASK)
#	define dd_LCMD	NX_DEVICELCMDKEYMASK
#elif defined(NX_NEXTLCMDKEYMASK)
#	define dd_LCMD	NX_NEXTLCMDKEYMASK
#endif
#if defined(NX_DEVICELALTKEYMASK)
#	define dd_LALT	NX_DEVICELALTKEYMASK
#elif defined(NX_NEXTLALTKEYMASK)
#	define dd_LALT	NX_NEXTLALTKEYMASK
#endif

#if defined(NX_DEVICERCTLKEYMASK)
#	define dd_RCTL	NX_DEVICERCTLKEYMASK
#endif
#if defined(NX_DEVICERSHIFTKEYMASK)
#	define dd_RSHFT	NX_DEVICERSHIFTKEYMASK
#elif defined(NX_NEXTRSHIFTKEYMASK)
#	define dd_RSHFT	NX_NEXTRSHIFTKEYMASK
#endif
#if defined(NX_DEVICERCMDKEYMASK)
#	define dd_RCMD	NX_DEVICERCMDKEYMASK
#elif defined(NX_NEXTRCMDKEYMASK)
#	define dd_RCMD	NX_NEXTRCMDKEYMASK
#endif
#if defined(NX_DEVICERALTKEYMASK)
#	define dd_RALT	NX_DEVICERALTKEYMASK
#elif defined(NX_NEXTRALTKEYMASK)
#	define dd_RALT	NX_NEXTRALTKEYMASK
#endif

#if defined(NX_STYLUSPROXIMITYMASK)
#	define dd_PROX	NX_STYLUSPROXIMITYMASK
#endif
#if defined(NX_STYLUSPROXIMITYMASK)
#	define dd_NONC	NX_NONCOALSESCEDMASK
#endif

static NSString *
modToString (unsigned mod)
{
	NSMutableString *str = [[NSMutableString alloc] init];
	unsigned res = mod;

	[str appendString:(mod & NSAlphaShiftKeyMask)	? @"@" : @"-"];	// (1 << 16)
	[str appendString:(mod & NSControlKeyMask)	? @"^" : @"-"];	// (1 << 18)
	[str appendString:(mod & NSShiftKeyMask)	? @"S" : @"s"];	// (1 << 17)
	[str appendString:(mod & NSCommandKeyMask)	? @"C" : @"c"];	// (1 << 20)
	[str appendString:(mod & NSAlternateKeyMask)	? @"A" : @"a"];	// (1 << 19)
	[str appendString:(mod & NSNumericPadKeyMask)	? @"#" : @"-"];	// (1 << 21)
	[str appendString:(mod & NSHelpKeyMask)		? @"H" : @"h"];	// (1 << 22)
	[str appendString:(mod & NSFunctionKeyMask)	? @"F" : @"f"];	// (1 << 23)
	res &= ~(NSAlphaShiftKeyMask|NSShiftKeyMask|NSControlKeyMask|NSAlternateKeyMask|
		 NSCommandKeyMask|NSNumericPadKeyMask|NSHelpKeyMask|NSFunctionKeyMask);

#if (defined(dd_LCTL) || defined(dd_LSHFT) || defined(dd_LCMD) || defined(dd_LALT))
	[str appendString:@"; L: "];
#endif
#ifdef dd_LCTL
	[str appendString:(mod & dd_LCTL)	? @"^" : @"-"];		// (1 << 0)
	res &= ~dd_LCTL;
#endif
#ifdef dd_LSHFT
	[str appendString:(mod & dd_LSHFT)	? @"S" : @"s"];		// (1 << 1)
	res &= ~dd_LSHFT;
#endif
#ifdef dd_LCMD
	[str appendString:(mod & dd_LCMD)	? @"C" : @"c"];		// (1 << 3)
	res &= ~dd_LCMD;
#endif
#ifdef dd_LALT
	[str appendString:(mod & dd_LALT)	? @"A" : @"a"];		// (1 << 5)
	res &= ~dd_LALT;
#endif

#if defined(dd_RCTL) || defined(dd_RSHFT) || defined(dd_RCMD) || defined(dd_RALT)
	[str appendString:@"; R: "];
#endif
#ifdef dd_RCTL
	[str appendString:(mod & dd_RCTL)	? @"^" : @"-"];		// (1 << 13)
	res &= ~dd_RCTL;
#endif
#ifdef dd_RSHFT
	[str appendString:(mod & dd_RSHFT)	? @"S" : @"s"];		// (1 << 2)
	res &= ~dd_RSHFT;
#endif
#ifdef dd_RCMD
	[str appendString:(mod & dd_RCMD)	? @"C" : @"c"];		// (1 << 4)
	res &= ~dd_RCMD;
#endif
#ifdef dd_RALT
	[str appendString:(mod & dd_RALT)	? @"A" : @"a"];		// (1 << 6)
	res &= ~dd_RALT;
#endif

#if defined(dd_PROX) || defined(dd_NONC)
	[str appendString:@"; O: "];
#endif
#ifdef dd_PROX
	[str appendString:(mod & dd_PROX)	? @"P" : @"p"];		// (1 << 7)
	res &= ~dd_PROX;
#endif
#ifdef dd_NONC
	[str appendString:(mod & dd_NONC)	? @"N" : @"n"];		// (1 << 8)
	res &= ~dd_NONC;
#endif

	if (res)
		[str appendFormat:@"; U: %#08lx", res];

	return str;
}

static NSString *
keyToString (unsigned len, unichar ch)
{
	NSMutableString *str = [[NSMutableString alloc] init];
	[str appendFormat:@"%u:(", len];

	if (ch < 32)
		[str appendFormat:@"^%c ", ch + '@'];
	else if (ch < 127)
		[str appendFormat:@"%c ", ch];
	else if (ch == 127)
		[str appendString:@"^? "];
	else if (ch < 256 && ch >= 128+32)
		[str appendFormat:@"%c ", ch];
	else if (ch >= NSF1FunctionKey && ch <= NSF35FunctionKey)
		[str appendFormat:@"F%u ", ch - NSF1FunctionKey + 1];
	else switch (ch) {
#define CASEfKs(key) #key
#define CASEfK(key) \
	case NS ##key## FunctionKey: \
		[str appendString:@"" CASEfKs(key) " "]; \
		break;
	CASEfK(UpArrow)
	CASEfK(DownArrow)
	CASEfK(LeftArrow)
	CASEfK(RightArrow)
	CASEfK(Insert)
	CASEfK(Delete)
	CASEfK(Home)
	CASEfK(Begin)
	CASEfK(End)
	CASEfK(PageUp)
	CASEfK(PageDown)
	CASEfK(PrintScreen)
	CASEfK(ScrollLock)
	CASEfK(Pause)
	CASEfK(SysReq)
	CASEfK(Break)
	CASEfK(Reset)
	CASEfK(Stop)
	CASEfK(Menu)
	CASEfK(User)
	CASEfK(System)
	CASEfK(Print)
	CASEfK(ClearLine)
	CASEfK(ClearDisplay)
	CASEfK(InsertLine)
	CASEfK(DeleteLine)
	CASEfK(InsertChar)
	CASEfK(DeleteChar)
	CASEfK(Prev)
	CASEfK(Next)
	CASEfK(Select)
	CASEfK(Execute)
	CASEfK(Undo)
	CASEfK(Redo)
	CASEfK(Find)
	CASEfK(Help)
	CASEfK(ModeSwitch)
	}

	[str appendFormat:@"%04x)", ch];

	return str;
}

@implementation EvKView
-(BOOL)isOpaque
{
	return YES;
}

-(BOOL)acceptsFirstResponder
{
	return YES;
}

-(void)drawRect:(NSRect)rect
{
	[[NSColor redColor] set];
	NSRectFill([self bounds]);
}

-(void)flagsChanged:(NSEvent *)anEvent
{
	unsigned mods = [anEvent modifierFlags];
	unsigned code = [anEvent keyCode];

	NSLog(@"mod: %@; key=%u", modToString(mods), code);
}

-(void)keyDown:(NSEvent *)anEvent
{
	unsigned mods = [anEvent modifierFlags];
	unsigned code = [anEvent keyCode];
	NSString *i_chars;
	NSString *m_chars;
	unsigned l_ich, l_mch;
	unichar ich, mch;
	char *repeat;

	m_chars = [anEvent characters];
	i_chars = [anEvent charactersIgnoringModifiers];
	l_mch = [m_chars length];
	l_ich = [i_chars length];
	repeat = [anEvent isARepeat] ? "; Repeat" : "";

	mch = l_mch ? [m_chars characterAtIndex:0] : 0xffff;
	ich = l_ich ? [i_chars characterAtIndex:0] : 0xffff;

	NSLog(@"key: %@; key=%u; ich %@; mch %@%s",
		modToString(mods), code,
		keyToString(l_ich, ich),
		keyToString(l_mch, mch),
		repeat);
}
@end
